@@ -472,33 +472,6 @@ foreach ($groups AS $group) {
 	$fuseKconfig .= $group->emitKconfig();
 }
 
-/*********************** fusecalc.h ***********************/
-
-$fusecalc_h  = "#ifndef _AVR_FUSECALC_".$DEVICE."_H\n";
-$fusecalc_h .= "#define _AVR_FUSECALC_".$DEVICE."_H_\n";
-$fusecalc_h .= "\n";
-$fusecalc_h .= "/* DO NOT EDIT. generated by decode_pack.php */\n";
-$fusecalc_h .= "\n";
-$fuse_bytes = [[], [], []];
-$used_fuse_bits = [0, 0, 0];
-foreach ($groups AS $group) {
-	if ($group->offset > 2) {
-		echo "ILLEGAL FUSE OFFSET $group->offset!\n";
-		exit(1);
-	}
-	$fusecalc_h .= $group->emitHeaderFile($fuse_bytes[$group->offset], $used_fuse_bits[$group->offset]);
-}
-for ($i = 0; $i < count($fuse_bytes); $i++) {
-	$mask = 0xff & ~$used_fuse_bits[$i];
-	$value = sprintf("0x%02x", $mask);
-	if (count($fuse_bytes[$i]))
-		$value = "(".$value." | ".implode(' | ', array_keys($fuse_bytes[$i])).")";
-	
-	$fusecalc_h .= "#define FUSE_BYTE_$i $value\n";
-}
-$fusecalc_h .= "\n";
-$fusecalc_h .= "#endif /* _AVR_FUSECALC_".$DEVICE."_H_ */\n";
-
 /*********************** irqvec.h ***********************/
 
 $interrupsNode = $doc->getElementsByTagName("interrupts")->item(0);
@@ -686,6 +659,7 @@ foreach ($otherModules AS $mod) {
 $ioregs_h .= "#endif /* _AVR_IOREGS_".$DEVICE."_H_ */\n";
 
 /*********************** limits.h ***********************/
+$num_fuse_bytes = 0;
 
 $memorySegmentNodes = $doc->getElementsByTagName("memory-segment");
 $limits_h  = "";
@@ -698,7 +672,7 @@ foreach ($memorySegmentNodes AS $child) {
 	if ($child->nodeName != "memory-segment")
 		continue;
 	$name = $child->getAttribute("name");
-	if ($name != "IRAM" && $name != "FLASH" && $name != "EEPROM")
+	if ($name != "IRAM" && $name != "FLASH" && $name != "EEPROM" && $name != "FUSES" && $name != "LOCKBITS")
 		continue;
 	if ($name == "IRAM")
 		$name = "RAM";
@@ -707,10 +681,13 @@ foreach ($memorySegmentNodes AS $child) {
 	$end_name = $name."_END";
 	$start_value = decode_number($child->getAttribute("start"));
 	$end_value = $start_value + decode_number($child->getAttribute("size")) - 1;
+	if ($name == "FUSES")
+		$num_fuse_bytes = $end_value + 1;
+
 	$limits_h .= sprintf("#define %-15s 0x%x\n", $start_name, $start_value);
 	$limits_h .= sprintf("#define %-15s 0x%x\n", $end_name, $end_value);
 	$limits_h .= "\n";
-	if ($name != "RAM") {
+	if ($name == "FLASH" || $name == "EEPROM") {
 		$pagesize = decode_number($child->getAttribute("pagesize"));
 		$limits_h .= sprintf("#define %-15s 0x%x\n", $name."_PAGESIZE", $pagesize);
 		$limits_h .= "\n";
@@ -718,6 +695,36 @@ foreach ($memorySegmentNodes AS $child) {
 }
 $limits_h .= "#endif /* _AVR_LIMITS_".$DEVICE."_H_ */\n";
 
+/*********************** fusecalc.h ***********************/
+
+$fusecalc_h  = "#ifndef _AVR_FUSECALC_".$DEVICE."_H\n";
+$fusecalc_h .= "#define _AVR_FUSECALC_".$DEVICE."_H_\n";
+$fusecalc_h .= "\n";
+$fusecalc_h .= "/* DO NOT EDIT. generated by decode_pack.php */\n";
+$fusecalc_h .= "\n";
+$fuse_bytes = [];
+$used_fuse_bits = [];
+for ($i = 0; $i < $num_fuse_bytes; $i++) {
+	$fuse_bytes[] = [];
+	$used_fuse_bits[] = 0;
+}
+foreach ($groups AS $group) {
+	if ($group->offset >= $num_fuse_bytes) {
+		echo "ILLEGAL FUSE OFFSET $group->offset!\n";
+		exit(1);
+	}
+	$fusecalc_h .= $group->emitHeaderFile($fuse_bytes[$group->offset], $used_fuse_bits[$group->offset]);
+}
+for ($i = 0; $i < $num_fuse_bytes; $i++) {
+	$mask = 0xff & ~$used_fuse_bits[$i];
+	$value = sprintf("0x%02x", $mask);
+	if (count($fuse_bytes[$i]))
+		$value = "(".$value." | ".implode(' | ', array_keys($fuse_bytes[$i])).")";
+	
+	$fusecalc_h .= "#define FUSE_BYTE_$i $value\n";
+}
+$fusecalc_h .= "\n";
+$fusecalc_h .= "#endif /* _AVR_FUSECALC_".$DEVICE."_H_ */\n";
 
 /*********************** Finally, write everything to the files ... ***********************/
 
