@@ -8,7 +8,7 @@
 
 #ifdef CONFIG_PRINTK
 
-extern void printk_(const char *fmt, ...);
+extern void printk_(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
 
 #define printk__(lvl, fmt, ...)  do { if (lvl <= KERN_LOGLEVEL) printk_(FMTSTR(fmt), ##__VA_ARGS__); } while(0)
 #define printk(fmt, ...) printk__(fmt, ##__VA_ARGS__)
